/**
 * Copyright (c) 2007-2015, Kaazing Corporation. All rights reserved.
 */

package com.kaazing.gateway.client;

import com.google.gwt.core.client.JavaScriptObject;
import com.google.gwt.core.client.JsArrayInteger;

public final class ByteBuffer extends JavaScriptObject {

    protected ByteBuffer() {}

    /**
     * Creates a new ByteBuffer instance.
     */
    public static native ByteBuffer create() /*-{
        return new $wnd.ByteBuffer();
    }-*/;
     
    /**
     * Creates a new ByteBuffer instance with bytes as underlying array.
     */
    public static native ByteBuffer create(JsArrayInteger bytes[]) /*-{
    	return new $wnd.ByteBuffer(bytes);
	}-*/;
 
    /**
     * The array property provides byte storage for the buffer.
     */
    public native JsArrayInteger getArray() /*-{
        return this.array;
    }-*/;

    public byte[] getBytes() {
    	byte[] dst = new byte[this.remaining()];
    	get(dst, this.position(), this.remaining());
    	return dst;
	}

    /**
     * The autoExpand property enables writing variable length data, and is on by default.
     */
    public native boolean getAutoExpand() /*-{
        return this.autoExpand;
    }-*/;

    /**
     * The capacity property indicates the maximum number of bytes of storage
     * available if the buffer is not automatically expanding.
     */
    public native int getCapacity() /*-{
        return this.capacity;
    }-*/;

    /**
     * The limit property indicates the last byte of data available for reading.
     */
    public native int limit() /*-{
    	return this.limit;
	}-*/;

    public native void limit(int newLimit) /*-{
		this.limit = newLimit;
	}-*/;

    /**
     * The order property indicates the endianness of multibyte integer types in the buffer.
     */
    public native ByteOrder getOrder() /*-{
        return this.order;
    }-*/;

    /**
     * The position property indicates the progress through the buffer, and indicates
     * the position within the underlying array that subsequent data will be read from
     * or written to.
     */
    public native int position() /*-{
    	return this.position;
	}-*/;

    public native void position(int newPosition) /*-{
		this.position = newPosition;
	}-*/;

    /**
     * Allocates a new ByteBuffer instance.
     */
    public static native ByteBuffer allocate(int capacity) /*-{
    	return new $wnd.ByteBuffer.allocate(capacity);
    }-*/;

    /**
     * Clears the buffer.
     */
    public native ByteBuffer clear() /*-{
        return this.clear();
    }-*/;

    /**
     * Compacts the buffer by removing leading bytes up to the buffer position, and
     * decrements the limit and position values accordingly.
     */
    public native ByteBuffer compact() /*-{
        return this.compact();
    }-*/;

    /**
     * Duplicates the buffer by reusing the underlying byte array but with independent
     * position, limit and capacity.
     */
    public native ByteBuffer duplicate() /*-{
        return this.duplicate();
    }-*/;

    /**
     * Expands the buffer to support the expected number of remaining bytes after the current position.
     */
    public native ByteBuffer expand(int expectedRemaining) /*-{
        return this.expand(expectedRemaining);
    }-*/;

    /**
     * Expands the buffer to support the expected number of remaining bytes at the specified index.
     */
    public native ByteBuffer expandAt(int i, int expectedRemaining) /*-{
        return this.expandAt(i, expectedRemaining);
    }-*/;

    /**
     * Fills the buffer with a repeated number of zeros.
     */
    public native ByteBuffer fill(int size) /*-{
        return this.fill(size);
    }-*/;

    /**
     * Fills the buffer with a specific number of repeated bytes.
     */
    public native ByteBuffer fillWith(byte b, int size) /*-{
        return this.fillWith(b, size);
    }-*/;

    /**
     * Flips the buffer.
     */
    public native ByteBuffer flip() /*-{
        return this.flip();
    }-*/;

    /**
     * Returns a single byte from the buffer at the current position.
     */
    public native byte get() /*-{
        return this.get();
    }-*/;
    
    public native byte[] getBytes(int size) /*-{
        return this.getBytes(size);
    }-*/;

    public void get(byte dst[], int offset, int size) {
		JsArrayInteger src = getArray();
    	int srcIndex = offset;
    	int dstIndex = 0;
    	while (dstIndex < size) {
			dst[dstIndex++] = (byte)src.get(srcIndex++);
		}
	}

    /**
     * Returns a single byte from the buffer at the specified index.
     */
    public native byte getAt(int i) /*-{
        return this.getAt(i);
    }-*/;

    /**
     * Returns a hex dump of this buffer.
     */
    public native String getHexDump() /*-{
        return this.getHexDump();
    }-*/;

    /**
     * Returns a 4-byte number from the buffer at the current position.
     */
    public native int getInt() /*-{
        return this.getInt();
    }-*/;

    /**
     * Returns a 4-byte number from the buffer at the specified index.
     */
    public native int getIntAt(int i) /*-{
        return this.getIntAt(i);
    }-*/;

    /**
     * Returns a 3-byte number from the buffer at the current position.
     */
    public native int getMediumInt() /*-{
        return this.getMediumInt();
    }-*/;

    /**
     * Returns a 3-byte number from the buffer at the specified index.
     */
    public native int getMediumIntAt(int i) /*-{
        return this.getMediumIntAt(i);
    }-*/;

    /**
     * Returns a length-prefixed string from the buffer at the current position.
     */
    public native String getPrefixedString(int fieldSize, Charset cs) /*-{
        return this.getPrefixedString(fieldSize, cs);
    }-*/;

    /**
     * Returns a 4-byte number from the buffer at the current position.
     */
    public native int getLong() /*-{
        return this.getLong();
    }-*/;


    /**
     * Returns a 2-byte number from the buffer at the current position.
     */
    public native short getShort() /*-{
        return this.getShort();
    }-*/;

    /**
     * Returns a 2-byte number from the buffer at the specified index.
     */
    public native int getShortAt(int i) /*-{
        return this.getShortAt(i);
    }-*/;

    /**
     * Returns a null-terminated string from the buffer at the current position.
     */
    public native String getString(Charset cs) /*-{
        return this.getString(cs);
    }-*/;

    /**
     * Returns an unsigned single-byte number from the buffer at the current position.
     */
    public native int getUnsigned() /*-{
        return this.getUnsigned();
    }-*/;

    /**
     * Returns an unsigned 4-byte number from the buffer at the current position.
     */
    public native int getUnsignedInt() /*-{
        return this.getUnsignedInt();
    }-*/;

    /**
     * Returns an unsigned 3-byte number from the buffer at the current position.
     */
    public native int getUnsignedMediumInt() /*-{
        return this.getUnsignedMediumInt();
    }-*/;

    /**
     * Returns an unsigned 2-byte number from the buffer at the current position.
     */
    public native int getUnsignedShort() /*-{
        return this.getUnsignedShort();
    }-*/;

    /**
     * Returns an ArrayBuffer from the buffer at the current position.
     */
    public native int getArrayBuffer(int size) /*-{
        return this.getUnsignedShort(size);
    }-*/;
    
    /**
     * Returns true if this buffer has remaining bytes, false otherwise.
     */
    public native boolean hasRemaining() /*-{
        return this.hasRemaining();
    }-*/;

    /**
     * Returns the index of the specified byte in the buffer.
     */
    public native int indexOf(byte b) /*-{
        return this.indexOf(b);
    }-*/;

    /**
     * Marks a position in the buffer.
     */
    public native ByteBuffer mark() /*-{
        return this.mark();
    }-*/;

    /**
     * Puts a single byte number into the buffer at the current position.
     */
    public native ByteBuffer put(byte v) /*-{
        return this.put(v);
    }-*/;

    /**
     * Puts a single byte number into the buffer at the specified index.
     */
    public native ByteBuffer putAt(int i, byte v) /*-{
        return this.putAt(i, v);
    }-*/;

    /**
     * Puts a buffer into the buffer at the current position.
     */
    public native ByteBuffer putBuffer(ByteBuffer v) /*-{
        return this.putBuffer(v);
    }-*/;

    /**
     * Puts a buffer into the buffer at the specified index.
     */
    public native ByteBuffer putBufferAt(int i, ByteBuffer v) /*-{
        return this.putBufferAt(i, v);
    }-*/;

    /**
     * Puts a single-byte array into the buffer at the current position.
     */
    public native ByteBuffer putBytes(byte[] v) /*-{
        return this.putBytes(v);
    }-*/;

    /**
     * Puts a byte array into the buffer at the specified index.
     */
    public native ByteBuffer putBytesAt(int i, byte[] v) /*-{
        return this.putBytesAt(i, v);
    }-*/;

    /**
     * Puts a 4-byte number into the buffer at the current position.
     */
    public native ByteBuffer putInt(int v) /*-{
        return this.putInt(v);
    }-*/;

    /**
     * Puts a 4-byte number into the buffer at the specified index.
     */
    public native ByteBuffer putIntAt(int i, int v) /*-{
        return this.putIntAt(i, v);
    }-*/;

    /**
     * Puts a 3-byte number into the buffer at the current position.
     */
    public native ByteBuffer putMediumInt(int v) /*-{
        return this.putMediumInt(v);
    }-*/;

    /**
     * Puts a 3-byte number into the buffer at the specified index.
     */
    public native ByteBuffer putMediumIntAt(int i, int v) /*-{
        return this.putMediumIntAt(i, v);
    }-*/;

    /**
     * Puts a string into the buffer at the specified index, using the character set to
     * encode the string as bytes.
     */
    public native ByteBuffer putPrefixedString(int fieldSize, int v, Charset cs) /*-{
        return this.putPrefixedString(fieldSize, v, cs);
    }-*/;

    /**
     * Puts a 8-byte number into the buffer at the current position.
     */
    public ByteBuffer putLong(long v) {
    	return this.putInt((int)v >> 32)
                   .putInt((int)v & ((1<<32)-1));
    }
    
    /**
     * Puts a 8-byte number into the buffer at the specified index.
     */
    public ByteBuffer putLongAt(int i, long v) {
    	return this.putIntAt(i,   (int)v >> 32)
                   .putIntAt(i+4, (int)v & ((1<<32)-1));
    }
    
    /**
     * Puts a 2-byte number into the buffer at the current position.
     */
    public native ByteBuffer putShort(short v) /*-{
        return this.putShort(v);
    }-*/;

    /**
     * Puts a 2-byte number into the buffer at the specified index.
     */
    public native ByteBuffer putShortAt(int i, short v) /*-{
        return this.putShortAt(i, v);
    }-*/;

    /**
     * Puts a two-byte array into the buffer at the current position.
     */
    public native ByteBuffer putUnsignedShort(short v) /*-{
        return this.putUnsignedShort(v);
    }-*/;
    
    /**
     * Puts a string into the buffer at the current position, using the character set to
     * encode the string as bytes.
     */
    public native ByteBuffer putString(String v, Charset cs) /*-{
        return this.putString(v, cs);
    }-*/;

    /**
     * Returns the number of bytes remaining from the current position to the limit.
     */
    public native int remaining() /*-{
        return this.remaining();
    }-*/;

    public native void remaining(int newRemaining) /*-{
    	this.remaining(newRemaining);
	}-*/;

    /**
     * Resets the buffer position using the mark.
     */
    public native ByteBuffer reset() /*-{
        return this.reset();
    }-*/;

    /**
     * Rewinds the buffer.
     */
    public native ByteBuffer rewind() /*-{
        return this.rewind();
    }-*/;

    /**
     * Skips the specified number of bytes from the current position.
     */
    public native ByteBuffer skip(int size) /*-{
        return this.skip(size);
    }-*/;

    /**
     * Returns a sliced buffer, setting the position to zero, and decrementing the limit accordingly.
     */
    public native ByteBuffer slice() /*-{
        return this.slice();
    }-*/;

    /**
     * Wraps a byte array as a new ByteBuffer instance.
     */
    public static native ByteBuffer wrap(byte[] bytes) /*-{
        return $wnd.ByteBuffer.wrap(bytes);
    }-*/;
}
